@@ -75,6 +75,16 @@ static void label(int x, int y, int w, int h, const char *text, NSView *view)
   [view addSubview:label];
 }
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
+const int radio_button_style = NSButtonTypeRadio;
+const int switch_button_style = NSButtonTypeSwitch;
+const int file_dialog_OK = NSModalResponseOK;
+#else
+const int radio_button_style = NSRadioButton;
+const int switch_button_style = NSSwitchButton;
+const int file_dialog_OK = NSFileHandlingPanelOKButton;
+#endif
+
 char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_only_button, const char *directory)
 {
   static char pathname[FL_PATH_MAX];
@@ -112,15 +122,15 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   NSSize cellsize= {80, 16};
   [pdfmatrix setCellSize:cellsize];
   NSButton *color = [[[NSButton alloc] init] autorelease];
-  [color setButtonType:NSRadioButton];
+  [color setButtonType:radio_button_style];
   [color setTitle:@"PDF color"];
   [pdfmatrix putCell:[color cell] atRow:0 column:0];
   NSButton *bw = [[[NSButton alloc] init] autorelease];
-  [bw setButtonType:NSRadioButton];
+  [bw setButtonType:radio_button_style];
   [bw setTitle:@"PDF B&W"];
   [pdfmatrix putCell:[bw cell] atRow:1 column:0];
   NSButton *textfile = [[[NSButton alloc] init] autorelease];
-  [textfile setButtonType:NSRadioButton];
+  [textfile setButtonType:radio_button_style];
   [textfile setTitle:@"Text File"];
   [pdfmatrix putCell:[textfile cell] atRow:2 column:0];
   [pdfmatrix selectCell:[bw cell]];
@@ -135,11 +145,11 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   [papermatrix setIntercellSpacing:spacing];
   [papermatrix setCellSize:cellsize];
   NSButton *A4 = [[[NSButton alloc] init] autorelease];
-  [A4 setButtonType:NSRadioButton];
+  [A4 setButtonType:radio_button_style];
   [A4 setTitle:@"A4"];
   [papermatrix putCell:[A4 cell] atRow:0 column:0];
   NSButton *LETTER = [[[NSButton alloc] init] autorelease];
-  [LETTER setButtonType:NSRadioButton];
+  [LETTER setButtonType:radio_button_style];
   [LETTER setTitle:@"LETTER"];
   [papermatrix putCell:[LETTER cell] atRow:1 column:0];
   [papermatrix selectCell:[LETTER cell]];
@@ -149,7 +159,7 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
 // landscape button
   NSRect landscaperect = {{280, 6}, {80, 20} };
   NSButton *landscape = [[[NSButton alloc] initWithFrame:landscaperect] autorelease];
-  [landscape setButtonType:NSSwitchButton];
+  [landscape setButtonType:switch_button_style];
   [landscape setTitle:@"landscape"];
   [landscape sizeToFit];
   [landscape setIntValue:(printout_layout == Fl_Paged_Device::LANDSCAPE)];
@@ -158,7 +168,7 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
 //variable site button  
   NSRect variablerect = {{360, 30}, {80, 20} };
   NSButton *variable = [[[NSButton alloc] initWithFrame:variablerect] autorelease];
-  [variable setButtonType:NSSwitchButton];
+  [variable setButtonType:switch_button_style];
   [variable setTitle:@"variable sites only"];
   [variable sizeToFit];
   [variable setIntValue:printout_vary];
@@ -180,7 +190,7 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   [key makeKeyWindow];
   [preset release];
   [dir release];
-  if ( retval == NSFileHandlingPanelOKButton ) {//read accessory view state
+  if ( retval == file_dialog_OK ) {//read accessory view state
     printout_block = [blockview intValue];
     printout_fontsize = [fontview intValue];
     printout_black = [pdfmatrix selectedRow];
@@ -190,8 +200,7 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
     strcpy(pathname, [[[_panel URL] path] UTF8String]);
     }
   [_panel setAccessoryView:nil];
-  if ( retval == NSFileHandlingPanelCancelButton ) return NULL;
-  return pathname;
+  return (retval == file_dialog_OK ? pathname : NULL);
 }
 
 #if 100*FL_MAJOR_VERSION + FL_MINOR_VERSION < 104
@@ -246,11 +255,6 @@ static void file_receive_cb(const char *fname)
 {
   SEA_VIEW *view;
   Fl_Window *w = Fl::first_window();
-#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION  + FL_PATCH_VERSION <= 131
-  // without this show() call, the opening of the alignment window is delayed by several seconds.
-  // not needed with FLTK 1.3.2 or above
-  if (fl_mac_os_version >= 100800 && w) w->show();
-#endif
   while(w != NULL) {
     const char *c = w->xclass();
     if(c != NULL && strcmp(c, SEAVIEW_WINDOW) == 0) break;
@@ -488,68 +492,27 @@ void Fl_PICT_Graphics_Driver::draw(const char* str, int n, float fx, float fy) {
   draw(str, n, (int)fx, (int)fy);
 }
 
-#endif // USE_PICT
-
-Copy_Surface::Copy_Surface(bool pict, int w, int h) : Fl_Surface_Device(NULL)
+Copy_Surface::Copy_Surface(int w, int h) : Fl_Surface_Device(NULL)
 {
-#ifdef USE_PICT
-  use_pict = pict;
-#else
-  use_pict = false;
-#endif
-  if (use_pict) {
-#ifdef USE_PICT
-      Rect myrect;
-      myrect.top=0;
-      myrect.left=0;
-      myrect.right=w;
-      myrect.bottom=h;
-      mypicture = OpenPicture(&myrect);
-      ClipRect(&myrect);
-      PenNormal();
-      driver(new Fl_PICT_Graphics_Driver());
-#endif
-    }
-  else {
-    prepare_copy_pdf_and_pict(w, h);
-#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION + FL_PATCH_VERSION >= 140
-    driver(Fl_Graphics_Driver::newMainGraphicsDriver());
-#else
-    driver(new Fl_Quartz_Graphics_Driver());
-#endif
-    oldgc = fl_gc;
-#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION + FL_PATCH_VERSION >= 140
-    driver()->gc(gc);
-#else
-    fl_gc = gc;
-#endif
-    }
+  Rect myrect;
+  myrect.top=0;
+  myrect.left=0;
+  myrect.right=w;
+  myrect.bottom=h;
+  mypicture = OpenPicture(&myrect);
+  ClipRect(&myrect);
+  PenNormal();
+  driver(new Fl_PICT_Graphics_Driver());
 }
 
 Copy_Surface::~Copy_Surface()
 {
-  if (use_pict) {
-#ifdef USE_PICT
-      ClosePicture();
-      MyCopyPictToClipboard();
-      KillPicture(mypicture);
-#endif
-    }
-  else {
-    complete_copy_pdf_and_pict();
-    fl_gc = oldgc;
-    }
+  ClosePicture();
+  MyCopyPictToClipboard();
+  KillPicture(mypicture);
   delete driver();
 }
 
-size_t Copy_Surface::MyPutBytes (void* info, const void* buffer, size_t count)
-{
-  CFDataAppendBytes ((CFMutableDataRef) info, (const UInt8 *)buffer, count);
-  return count;
-}
-
-
-#ifdef USE_PICT
 QDPictRef Copy_Surface::MyPictToQDPict()
 {
   CFIndex pictlen;
@@ -564,7 +527,6 @@ QDPictRef Copy_Surface::MyPictToQDPict()
 	  CFRelease(provider);
 	  return myqdpict;
   }
-  else return NULL;
   return NULL;
 }
 
@@ -574,7 +536,6 @@ void Copy_Surface::MyCopyPictToClipboard()
   CFDataRef  data = NULL;
   PasteboardRef clipboard = NULL;
   CFIndex pictlen;
-  QDPictRef myqdpict;	
   
   pictlen = GetHandleSize( (Handle)  mypicture);
   PasteboardCreate (kPasteboardClipboard, &clipboard);
@@ -587,12 +548,15 @@ void Copy_Surface::MyCopyPictToClipboard()
 				      data,
 				      kPasteboardFlavorNoFlags );
   CFRelease (data);
-  if (fl_mac_os_version < 100400) return; // useful under 10.3 for unclear reason
-  
-  myqdpict = MyPictToQDPict();
+  QDPictRef myqdpict = (fl_mac_os_version >= 100400 ? MyPictToQDPict() : 0); // useful under 10.3 for unclear reason
+  if (!myqdpict) {
+    CFRelease(clipboard);
+    return;
+  }
   
   CGRect bounds = QDPictGetBounds (myqdpict);
-  init_PDF_context(bounds.size.width, bounds.size.height);
+  CFMutableDataRef pdfdata;
+  CGContextRef gc = init_PDF_context(bounds.size.width, bounds.size.height, pdfdata);
   if (gc) {
     /* convert PICT to PDF */
     CGContextBeginPage (gc, &bounds);
@@ -603,104 +567,196 @@ void Copy_Surface::MyCopyPictToClipboard()
     (void) PasteboardPutItemFlavor (clipboard, (PasteboardItemID)1, 
 				     /* kUTTypePDF */ CFSTR("com.adobe.pdf"), 
 				     pdfdata, kPasteboardFlavorNoFlags);
+    CFRelease (pdfdata);
   }
-  CFRelease (pdfdata); 
   CFRelease (clipboard);
-  if(myqdpict != NULL) QDPictRelease(myqdpict);
-}
-#endif // USE_PICT
-
-void Copy_Surface::init_PDF_context(int w, int h)
-{
-  CGRect bounds = CGRectMake(0, 0, w, h );	
-  pdfdata = CFDataCreateMutable(NULL, 0);
-  CGDataConsumerRef myconsumer;
-  if (CGDataConsumerCreateWithCFData) { // true from 10.4
-    myconsumer = CGDataConsumerCreateWithCFData (pdfdata);
-  }
-  else {
-    static CGDataConsumerCallbacks callbacks = { MyPutBytes, NULL };
-    myconsumer = CGDataConsumerCreate ((void*) pdfdata, &callbacks);
-  }
-  gc = CGPDFContextCreate (myconsumer, &bounds, NULL);
-  CGDataConsumerRelease (myconsumer);
+  QDPictRelease(myqdpict);
 }
 
-//next 2 functions copy quartz data to clipboard both as pdf and as tiff bitmap
-void Copy_Surface::prepare_copy_pdf_and_pict(int w, int h)
-{
-  init_PDF_context(w, h);
-  if (gc == NULL) return;
-  CGRect bounds = CGRectMake(0, 0, w, h );	
-  CGContextBeginPage (gc, &bounds);
-  CGContextTranslateCTM(gc, 0, h);
-  CGContextScaleCTM(gc, 1.0f, -1.0f);
-}
 
-void Copy_Surface::complete_copy_pdf_and_pict()
+CGContextRef Copy_Surface::init_PDF_context(int w, int h, CFMutableDataRef &pdfdata)
 {
-  int w, h;
-  CGContextEndPage (gc);
-  CGContextRelease (gc);
-  PasteboardRef clipboard = NULL;
-  PasteboardCreate (kPasteboardClipboard, &clipboard);
-  PasteboardClear(clipboard); //	copy PDF to clipboard 
-  PasteboardPutItemFlavor (clipboard, (PasteboardItemID)1, 
-						   CFSTR("com.adobe.pdf"), // kUTTypePDF
-						   pdfdata, kPasteboardFlavorNoFlags);
-  //second, transform this PDF to a bitmap image and put it as tiff in clipboard
-  CGDataProviderRef prov;
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1040
-  if(fl_mac_os_version >= 100400)
-    prov = CGDataProviderCreateWithCFData(pdfdata); // 10.4
-  else 
+  CGRect bounds = CGRectMake(0, 0, w, h );
+  pdfdata = CFDataCreateMutable(NULL, 0);
+  if (&CGDataConsumerCreateWithCFData) { // true from 10.4
+    CGDataConsumerRef myconsumer = CGDataConsumerCreateWithCFData (pdfdata);
+    CGContextRef gc = CGPDFContextCreate (myconsumer, &bounds, NULL);
+    CGDataConsumerRelease (myconsumer);
+    return gc;
+  }
 #endif
-    prov = CGDataProviderCreateWithData(NULL, CFDataGetBytePtr(pdfdata), CFDataGetLength(pdfdata), NULL);
-  CGPDFDocumentRef pdfdoc = CGPDFDocumentCreateWithProvider(prov);
-  CGPDFPageRef pdfpage = CGPDFDocumentGetPage(pdfdoc, 1);
-  CGRect clip = CGPDFPageGetBoxRect(pdfpage, kCGPDFCropBox);
-  w = (int)(clip.size.width + 0.5);
-  h = (int)(clip.size.height + 0.5);
-  CGDataProviderRelease(prov);
-  CGColorSpaceRef space = CGColorSpaceCreateDeviceRGB();
-  const int scale = 2;
-  w *= scale; h *= scale;
-  void *mem = ( fl_mac_os_version >= 100600 ? NULL : malloc(w * h * 4) );
-  gc = CGBitmapContextCreate(mem, w, h, 8, w * 4, space, kCGImageAlphaNoneSkipLast );
-  CFRelease(space);
-  if (gc == NULL) { if (mem) free(mem); CGPDFDocumentRelease(pdfdoc); return; }
-  CGRect rect = CGRectMake(0, 0, w, h);
-  CGContextSetRGBFillColor(gc,  1,1,1,1);//need to clear background
-  CGContextFillRect(gc, rect);
-  CGContextConcatCTM(gc, CGAffineTransformMakeScale(scale, scale));
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1030
-  CGContextDrawPDFPage(gc, pdfpage);  // 10.3
-#endif
-  CGPDFDocumentRelease(pdfdoc);
-  CFRelease(pdfdata);
-  unsigned char *plane = (unsigned char *)CGBitmapContextGetData(gc);
-  NSBitmapImageRep *rep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:&plane
-								  pixelsWide:w
-								  pixelsHigh:h
-							       bitsPerSample:8
-							     samplesPerPixel:3
-								    hasAlpha:NO
-								    isPlanar:NO
-							      colorSpaceName:NSDeviceRGBColorSpace
-								 bytesPerRow:w*4
-								bitsPerPixel:32];
-  CFDataRef data = (CFDataRef)[rep TIFFRepresentation];
-  [rep release];
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1030
-  PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, CFSTR("public.tiff"), data, kPasteboardFlavorNoFlags );
-#endif
-  CFRelease (clipboard);    
-  CGContextRelease(gc);
-  if (mem) free(mem);
+  return NULL;
 }
 
+#endif // USE_PICT
+
 void mac_tmp_dir_name(char *buffer, int l)
 {
   CFStringGetCString((CFStringRef)NSTemporaryDirectory(), buffer, l, kCFStringEncodingUTF8);
 }
+
+#ifdef SAVE_WINDOW_AS_EPS
+
+#include <FL/Fl_PostScript.H>
+
+class EPS_File_Surface : public Fl_Widget_Surface {
+protected:
+  inline Fl_PostScript_Graphics_Driver *driver() { return (Fl_PostScript_Graphics_Driver*)Fl_Surface_Device::driver(); }
+public:
+  EPS_File_Surface(const char *filename, int width, int height, Fl_Color background = FL_WHITE);
+  ~EPS_File_Surface();
+  int start_eps(int width, int height);
+  virtual int printable_rect(int *w, int *h);
+  FILE *file() { return driver()->output; }
+  virtual void origin(int x, int y);
+  virtual void origin(int *px, int *py);
+  virtual void translate(int x, int y);
+  virtual void untranslate();
+};
+
+EPS_File_Surface::EPS_File_Surface(const char *filename, int width, int height, Fl_Color background) :
+        Fl_Widget_Surface(new Fl_PostScript_Graphics_Driver()) {
+  Fl_PostScript_Graphics_Driver *ps = driver();
+  ps->output = fl_fopen(filename, "w+");
+  if (ps->output) {
+    ps->ps_filename_ = strdup(filename);
+    float s = Fl::screen_scale(0);
+    start_eps(width*s, height*s);
+    if (s != 1) {
+      ps->clocale_printf("GR GR GS %f %f SC GS\n", s, s);
+      ps->scale_x = ps->scale_y = s;
+    }
+    Fl::get_color(background, ps->bg_r, ps->bg_g, ps->bg_b);
+  }
+}
+
+EPS_File_Surface::~EPS_File_Surface() {
+  Fl_PostScript_Graphics_Driver *ps = driver();
+  if(ps->output == NULL) return;
+  fputs("GR\nend %matches begin of FLTK dict\n", ps->output);
+  fputs("restore\n", ps->output);
+  fputs("%%EOF\n", ps->output);
+  ps->reset();
+  fflush(ps->output);
+  if(ferror(ps->output)) {
+    fl_alert ("Error during PostScript data output.");
+  }
+  fclose(ps->output);
+  while (ps->clip_){
+    Fl_PostScript_Graphics_Driver::Clip * c= ps->clip_;
+    ps->clip_= ps->clip_->prev;
+    delete c;
+  }
+}
+
+int EPS_File_Surface::start_eps(int width, int height) {
+  Fl_PostScript_Graphics_Driver *ps = driver();
+  ps->width_ = width;
+  ps->height_ = height;
+  fputs("%!PS-Adobe-3.0 EPSF-3.0\n", file());
+  fputs("%%Creator: (FLTK)\n", file());
+  fprintf(file(),"%%%%BoundingBox: 1 1 %d %d\n", width, height);
+  if (ps->ps_filename_) fprintf(file(),"%%%%Title: (%s)\n", fl_filename_name(ps->ps_filename_));
+  time_t lt = time(NULL);
+  fprintf(file(),"%%%%CreationDate: %s", ctime(&lt)+4);
+  ps->lang_level_= 2;
+  fprintf(file(), "%%%%LanguageLevel: 2\n");
+  fputs("%%Pages: 1\n%%EndComments\n", file());
+  fputs("%%BeginProlog\n", file());
+  fputs("%%EndProlog\n",file());
+  fprintf(file(), "save\n");
+  fputs("/FLTK 20 dict def FLTK begin\n"
+  "/x1 0 def /x2 0 def /y1 0 def /y2 0 def /x 0 def /y 0 def /dx 0 def /dy 0 def\n"
+        "/px 0 def /py 0 def /sx 0 def /sy 0 def /inter 0 def\n"
+        "/pixmap_sx 0 def  /pixmap_sy 0 def /pixmap_w 0 def /pixmap_h 0 def\n", file());
+  // trick to get the prolog
+  long cur_pos = ftell(file());
+  Fl_PostScript_File_Device *tmp = new Fl_PostScript_File_Device();
+  tmp->begin_job(file());
+  tmp->end_job();
+  delete tmp;
+  fseek(file(), cur_pos, SEEK_SET);
+  char line[200];
+  do fgets(line, sizeof(line), file());
+  while(strncmp(line, "%%BeginProlog\n", 14));
+  char *prolog = new char[10000], *p = prolog;
+  while (1) {
+    fgets(line, sizeof(line), file());
+    if (strncmp(line, "/CS { GS } bind def\n", 20) == 0) break;
+    strcpy(p, line);
+    p += strlen(p);
+  }
+  int lprolog = strlen(prolog);
+  fseek(file(), cur_pos, SEEK_SET);
+  fwrite(prolog, 1, lprolog, file());
+  delete[] prolog;
+  // end of trick
+  fputs("/CS { GS } bind def\n", file());
+  fputs("/CR { GR } bind def\n", file());
+  ps->page_policy_ = 1;
+  ps->reset();
+  ps->nPages=0;
+  fprintf(file(), "GS\n");
+  ps->clocale_printf( "%g %g TR\n", (double)0, ps->height_);
+  fprintf(file(), "1 -1 SC\n");
+  ps->line_style(0);
+  fprintf(file(), "GS GS\n");
+  return 0;
+}
+
+int EPS_File_Surface::printable_rect(int *w, int *h) {
+  Fl_PostScript_Graphics_Driver *ps = driver();
+  *w = ps->width_;
+  *h = ps->height_;
+  return 0;
+}
+
+void EPS_File_Surface::origin(int x, int y)
+{
+  x_offset = x;
+  y_offset = y;
+  driver()->clocale_printf("GR GR GS %d %d TR  %f %f SC %d %d TR %f rotate GS\n",
+    0, 0, driver()->scale_x, driver()->scale_y, x, y, 0);
+}
+
+void EPS_File_Surface::origin(int *px, int *py) {
+  Fl_Widget_Surface::origin(px, py);
+}
+
+void EPS_File_Surface::translate(int x, int y)
+{
+  fprintf(file(), "GS %d %d translate GS\n", x, y);
+}
+
+void EPS_File_Surface::untranslate()
+{
+  fprintf(file(), "GR GR\n");
+}
+
+void EPS_cb(Fl_Widget *, void *data) {
+  Fl_Native_File_Chooser fnfc;
+  fnfc.title("Pick an .eps file");
+  fnfc.type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
+  fnfc.filter("EPS\t*.eps\n");
+  if (!fnfc.show() ) {
+    Fl_Window *target = Fl::first_window();
+    if (target) {
+      EPS_File_Surface p(fnfc.filename(), target->decorated_w(), target->decorated_h());
+      if (p.file()) {
+        p.draw_decorated_window(target);
+      }
+    }
+  }
+}
+
+static Fl_Menu_Item custom[] = {
+  { "Save front window as EPS", 0, EPS_cb, 0, 0 },
+  { 0 }
+};
+
+static int unused = (Fl_Mac_App_Menu::custom_application_menu_items(custom), 0);
+
+#endif // SAVE_WINDOW_AS_EPS
+
 #endif //__APPLE__
